/* $Id: ikev2_impl.h,v 1.10 2005/06/21 08:39:20 fukumoto Exp $ */

/*
 * Copyright (C) 2004-2005 WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#define	IKEV2_DEFAULT_RETRY	10
#define	IKEV2_DEFAULT_NEGOTIATION_TIMEOUT	600 /* ??? */
#define	IKEV2_DEFAULT_LIFETIME_TIME	0 /* indefinite */
#define	IKEV2_DEFAULT_POLLING_INTERVAL	3600 /* 1hour */

/* (draft-ietf-ipsec-esp-v3-09.txt)
   A minimum window size of 32 packets MUST be supported when 32-bit
   sequence numbers are employed; a window size of 64 is preferred and
   SHOULD be employed as the default.
*/
#ifdef __linux__
/* reading USAGI linux26/net/xfrm code, it seems that:
 *   it does not check window size value, and
 *   the replay check is done as:
 *     if diff >= window size  then return -EINVAL
 *     if x->replay.bitmap & (1U << diff) then return -EINVAL
 *   where replay.bitmap is type __u32.
 *   which means it might pass packet erroneously if window size is >32 and diff >= 32
 * thus window size MUST be <= 32
 */
/* use 32 for now */
#define	IKEV2_IPSEC_WINDOW_SIZE	32
#else
#define	IKEV2_IPSEC_WINDOW_SIZE	64
#endif

extern int ikev2_ipsec_window_size;


/*
 * (2.10) Nonces used in IKEv2
 * MUST be randomly chosen, MUST be at least 128 bits in size, and MUST
 * be at least half the key size of the negotiated prf.
 *
 * (3.9) The size of a Nonce MUST be between 16 and 256 octets inclusive.
 *
 */
/* so far, HMAC TIGER's 192bits output is the longest defined for prf */
/* thus 256 bits (32 octets) shall be adequate */
#define	IKEV2_DEFAULT_NONCE_SIZE		(256 / 8)


extern struct ikev2_payload_types {
    char	* name;
    size_t	minimum_length;
} ikev2_payload_types[];

#define	IKEV2_PAYLOAD_TYPE_DEFINED(type_)	((type_) >= IKEV2_PAYLOAD_SA && (type_) <= IKEV2_PAYLOAD_EAP)
#define	IKEV2_PAYLOAD_TYPES(type_)	(ikev2_payload_types[(type_) - IKEV2_PAYLOAD_SA])
#define	IKEV2_PAYLOAD_NAME(type_)	(IKEV2_PAYLOAD_TYPE_DEFINED(type_) ? IKEV2_PAYLOAD_TYPES(type_).name : "unknown")



enum ikev2_state {
    IKEV2_STATE_IDLING = 0,
    IKEV2_STATE_INI_IKE_SA_INIT_SENT = 1,
    IKEV2_STATE_RES_IKE_SA_INIT_SENT = 2,
    IKEV2_STATE_INI_IKE_AUTH_SENT = 3,
    IKEV2_STATE_ESTABLISHED = 4,
    IKEV2_STATE_DYING = 5,
    /* IKEV2_STATE_DEAD = 6, */
    /* IKEV2_STATE_EAP  = 7 */

    /* IKEV2_STATE_ESTABLISHED_WAIT_INITIATOR, */
};


enum ikev2_child_state {
    IKEV2_CHILD_STATE_IDLING = 0,
    IKEV2_CHILD_STATE_GETSPI,
    IKEV2_CHILD_STATE_GETSPI_DONE,
    IKEV2_CHILD_STATE_WAIT_RESPONSE,
    IKEV2_CHILD_STATE_MATURE,
    IKEV2_CHILD_STATE_EXPIRED,	/* XXX STATE_DONE */
    IKEV2_CHILD_STATE_INFO_PENDING,
    IKEV2_CHILD_STATE_INFO_SENT,
    IKEV2_CHILD_STATE_NUM,

    IKEV2_CHILD_STATE_INVALID	/* to indicate invalid state */
};


#define	IKEV2_SA_LIST_HEAD		TAILQ_HEAD(ikev2_sa_list, ikev2_sa)
#define	IKEV2_SA_LIST_INIT(head_)	TAILQ_INIT(head_)
#define	IKEV2_SA_LIST_ENTRY		TAILQ_ENTRY(ikev2_sa)
#define	IKEV2_SA_LIST_FIRST(head_)	TAILQ_FIRST(head_)
#define	IKEV2_SA_LIST_NEXT(p_)		TAILQ_NEXT((p_), link)
#define	IKEV2_SA_LIST_END(p_)		((p_) == 0)
#define	IKEV2_SA_LIST_LINK(head_, p_)	TAILQ_INSERT_TAIL((head_), (p_), link)
#define	IKEV2_SA_LIST_REMOVE(head_, p_)	TAILQ_REMOVE((head_), (p_), link)


#define	IKEV2_CHILD_LIST_HEAD			TAILQ_HEAD(ikev2_child_sa_list, ikev2_child_sa)
#define	IKEV2_CHILD_LIST_INIT(head_)		TAILQ_INIT(head_)
#define	IKEV2_CHILD_LIST_EMPTY(head_)		TAILQ_EMPTY((head_))
#define	IKEV2_CHILD_LIST_ENTRY			TAILQ_ENTRY(ikev2_child_sa)
#define	IKEV2_CHILD_LIST_FIRST(head_)		TAILQ_FIRST((head_))
#define	IKEV2_CHILD_LIST_NEXT(p_)		TAILQ_NEXT((p_), link)
#define	IKEV2_CHILD_LIST_END(p_)		((p_) == 0)
#define	IKEV2_CHILD_LIST_LINK(head_, p_)	TAILQ_INSERT_TAIL((head_), (p_), link)
#define	IKEV2_CHILD_LIST_REMOVE(head_, p_)	TAILQ_REMOVE((head_), (p_), link)


struct ikev2_sa {
    isakmp_index_t	index;

#ifdef DEBUG_TRACE
    int			serial_number; /* for tracking */
#endif
    int			version;
    int			is_initiator; /* side */

    struct sockaddr	* remote;
    struct sockaddr	* local;

    struct rcf_remote	* rmconf;

    u_int32_t		send_message_id; /* for sending request */
    int			request_pending; /* number of requests in progress */
    u_int32_t		recv_message_id; /* for receiving request */

    int			state;	/* enum STATE */

    /* int			got_response;  */
    /* struct timeval	last_received_time; */

    /* (draft-ietf-ipsec-ikev2-12.txt)
     * If a cryptographically protected message has been received from
     * the other side recently, unprotected notifications MAY be
     * ignored.  Implementations MUST limit the rate at which they
     * take actions based on unprotected messages.
     */

    struct ikev2_isakmpsa	* negotiated_sa; /* XXX approval */

    struct keyed_hash	* prf;	/* negotiated prf */

    vchar_t		* n_i;		/* Ni */
    vchar_t		* n_r;		/* Nr */

    struct algdef	* dh_choice;	/* to retry with different DH group */

    vchar_t		* dhpriv; 	/* x	*/
    vchar_t		* dhpub;	/* g^x	*/
    vchar_t		* dhpub_p;	/* g^y  */

    vchar_t		* dhgxy;	/* g^xy */

    vchar_t		* skeyseed;
    vchar_t		* sk_d;
    vchar_t		* sk_a_i;
    vchar_t		* sk_a_r;
    vchar_t		* sk_e_i;
    vchar_t		* sk_e_r;
    vchar_t		* sk_p_i;
    vchar_t		* sk_p_r;

    vchar_t		* id_i;	/* IDi' */
    vchar_t		* id_r;	/* IDr' */

    vchar_t		* my_first_message; /* for AUTH calculation */
    vchar_t		* peer_first_message;

    struct encryptor	* encryptor;
    struct authenticator * authenticator;

    /* pending_requests; */ /* CREATE_CHILD_SA, INFORMATIONAL */
    IKEV2_CHILD_LIST_HEAD	children;

    struct transmit_info transmit_info;
				/* XXX should have two transmit_info:
				   one for sending requests and one for
				   sending responses */

    struct timeval	due_time; /* certificate expiration time */
    struct sched	* expire_timer;
    struct sched	* polling_timer;
    size_t		lifetime_byte;

    IKEV2_SA_LIST_ENTRY	link;
};


/* negotiated parameters */
struct ikev2_child_param {
    int		use_transport_mode; /* IN */
    int		esp_tfc_padding_not_supported; /* IN */
    int		single_pair_required; /* OUT */
    int		additional_ts_possible;	/* OUT */
    vchar_t	* ts_i;		/* OUT */
    vchar_t	* ts_r;		/* OUT */
    u_int	notify_code;	/* OUT */
};


struct ikev2_child_sa {
    int				is_initiator; /* whether this side initiated the child_sa negotiation */
    enum ikev2_child_state	state;

    struct sockaddr		* local;
    struct sockaddr		* remote;

    u_int32_t			message_id;
    /* struct timeval		time; */

    struct sadb_request		sadb_request;

    vchar_t			* n_i;
    vchar_t			* n_r;

    vchar_t			* ts_i;
    vchar_t			* ts_r;

    struct rcf_selector		* selector;
    struct prop_pair		** my_proposal;
    struct prop_pair		* peer_proposal;

    struct sched		* timer; /* expiration timer */

    IKEV2_CHILD_LIST_ENTRY	link;
    struct ikev2_sa		* parent;

    struct ikev2_child_param	child_param;

    /* for informational exchange */
    void			(* callback)(int, struct ikev2_child_sa *, void *);
    void			* callback_param;
};


enum info_callback {
    INFORMATIONAL_CALLBACK_CONTINUE,
    INFORMATIONAL_CALLBACK_TRANSMIT_ERROR,
    INFORMATIONAL_CALLBACK_RESPONSE,
};


/*
 * packet construction utilities
 */
struct ikev2_payload_info {
    unsigned int	type;
    vchar_t		* data;
    int			need_free;
};

struct ikev2_payloads {
    int		num;
    struct ikev2_payload_info	* payloads;
};

extern void ikev2_payloads_init(struct ikev2_payloads *p);
extern void ikev2_payloads_push(struct ikev2_payloads *p, int type, vchar_t *data, int need_free);
extern void ikev2_payloads_destroy(struct ikev2_payloads *p);
extern vchar_t *ikev2_packet_construct(int exch_type, int flags, u_int32_t message_id,
				       struct ikev2_sa *ike_sa, struct ikev2_payloads *payl_list);


extern void ikev2_respond_with_notify(vchar_t *request,
				      struct sockaddr *remote,
				      struct sockaddr *local,
				      int msg_type,
				      u_int8_t *nmsg, int nmsg_len);

extern void ikev2_update_message_id(struct ikev2_sa *ike_sa, u_int32_t message_id, int is_response);
extern int ikev2_transmit_response(struct ikev2_sa *ike_sa, vchar_t *packet, struct sockaddr *src, struct sockaddr *dst);

extern int ikev2_respond_null(struct ikev2_sa *ike_sa, vchar_t *request, struct sockaddr *remote, struct sockaddr *local);
extern int ikev2_respond_error(struct ikev2_sa *ike_sa, vchar_t *request, struct sockaddr *remote, struct sockaddr *local,
				u_int protocol_id, u_int8_t *spi, int spi_size,
				u_int notify_type, void *data, size_t datalen);

extern void ikev2_poll(struct ikev2_sa *sa);
extern void ikev2_sa_start_polling_timer(struct ikev2_sa *sa);

extern int ikev2_check_spi_size(struct isakmp_domain *doi, int proto, int size);
extern int ikev2_ts_addr_size(int type);

extern void ikev2_set_state(struct ikev2_sa *sa, int state);

extern void ikev2_responder_state1_send(struct ikev2_sa *ike_sa, struct ikev2_child_sa *child_sa);

extern int ikev2_createchild_initiator_send(struct ikev2_sa *ike_sa, struct ikev2_child_sa *child_sa);
extern void ikev2_createchild_responder_recv(struct ikev2_sa *ike_sa, vchar_t *msg, struct sockaddr *remote, struct sockaddr *local);
extern void ikev2_createchild_responder_send(struct ikev2_sa *ike_sa, struct ikev2_child_sa *child_sa);
extern void ikev2_createchild_initiator_recv(struct ikev2_sa *ike_sa, vchar_t *msg, struct sockaddr *remote, struct sockaddr *local);


extern void informational_initiator_notify(struct ikev2_sa *ike_sa, struct ikev2_payloads *payl);

extern void informational_initiator_delete(struct ikev2_sa *ike_sa, struct ikev2_payloads *payl);

extern int ikev2_send_initial_contact(struct ikev2_sa *ike_sa);

extern void ikev2_sa_init(void);
extern struct ikev2_sa *ikev2_find_sa(vchar_t *message);
extern struct ikev2_sa * ikev2_find_sa_by_addr(struct sockaddr *addr);
extern struct ikev2_sa *ikev2_create_sa(isakmp_cookie_t *initiator_spi, struct sockaddr *local, struct sockaddr *remote, struct rcf_remote *conf);
extern void ikev2_sa_stop_timer(struct ikev2_sa *sa);
extern void ikev2_sa_start_lifetime_timer(struct ikev2_sa *sa);
extern void ikev2_sa_expire(struct ikev2_sa *sa);
extern void ikev2_dispose_sa(struct ikev2_sa *sa);
extern struct ikev2_child_sa * ikev2_create_child_initiator(struct ikev2_sa *ike_sa);
extern struct ikev2_child_sa * ikev2_create_child_responder(struct ikev2_sa *ike_sa,
							    struct sockaddr *local,
							    struct sockaddr *remote,
							    u_int32_t request_message_id,
							    struct ikev2_payload_header *sa_i2,
							    struct ikev2_payload_header *proposed_ts_i,
							    struct ikev2_payload_header *proposed_ts_r,
							    vchar_t *n_i,
							    struct ikev2_child_param *child_param);
extern int ikev2_set_negotiated_sa(struct ikev2_sa *ike_sa, struct ikev2_isakmpsa *negotiated_sa);
extern void ikev2_set_rmconf(struct ikev2_sa *sa, struct rcf_remote *conf);
extern struct rc_idlist *ikev2_my_id_list(struct ikev2_sa *ike_sa);

extern struct encryptor * ikev2_encryptor_new(int alg, int klen);
extern struct authenticator * ikev2_authenticator_new(int alg);
extern struct keyed_hash * ikev2_prf_new(int alg);

extern void ikev2_child_state_set(struct ikev2_child_sa *child_sa, enum ikev2_child_state state);
extern void ikev2_child_state_next(struct ikev2_child_sa *child_sa);

extern struct ikev2_child_sa * ikev2_create_child_sa(struct ikev2_sa *ike_sa);
extern void ikev2_destroy_child_sa(struct ikev2_child_sa *sa);
extern void ikev2_insert_child(struct ikev2_sa *ike_sa, struct ikev2_child_sa *child_sa);
extern void ikev2_remove_child(struct ikev2_child_sa *child_sa);
extern struct ikev2_child_sa * ikev2_find_child_sa_by_spi(struct ikev2_sa *ike_sa,
							  u_int protocol_id,
							  u_int32_t spi);

extern struct ikev2_child_sa * ikev2_choose_pending_child(struct ikev2_sa *ike_sa, int may_not_be_createchild);
extern struct ikev2_child_sa * ikev2_find_child_sa(struct ikev2_sa *ike_sa, int is_responder, u_int32_t id);
extern void ikev2_wakeup_child_sa(struct ikev2_child_sa *child_sa);

extern void ikev2_update_child(struct ikev2_child_sa *child_sa,
			       struct ikev2_payload_header *sa_r2,
			       struct ikev2_payload_header *ts_i,
			       struct ikev2_payload_header *ts_r,
			       struct ikev2_child_param *param);

void ikev2_expired(struct sadb_request *req, u_int satype, u_int32_t spi,
		   struct sockaddr *src, struct sockaddr *dst,
		   u_int samode, int expired);

void ikev2_abort(struct ikev2_sa *ike_sa, int err);
void ikev2_child_abort(struct ikev2_child_sa *child_sa, int err);

extern int ikev2_child_getspi(struct ikev2_child_sa * child_sa);
extern int ikev2_child_getspi_response(struct sadb_request *req,
				       struct sockaddr *src,
				       struct sockaddr *dst,
				       u_int satype, u_int32_t spi);

extern vchar_t * ikev2_pack_proposal(struct prop_pair **proposal);




vchar_t * ikev2_encrypt(struct ikev2_sa *ike_sa, vchar_t *payloads);

void ikev2_rekey_ikesa_responder(vchar_t *request,
				 struct sockaddr *remote,
				 struct sockaddr *local,
				 struct ikev2_sa *old_sa,
				 struct ikev2_payload_header *sa_payload,
				 struct ikev2_payload_header *ke,
				 struct ikev2_payload_header *nonce);

int ikev2_check_payloads(vchar_t *packet, int before_decrypt);
int ikev2_check_ts_payload(struct ikev2_payload_header *payload);
int ikev2_check_icv(struct ikev2_sa *ike_sa, vchar_t *packet);
int ikev2_decrypt(struct ikev2_sa *ike_sa, vchar_t *packet);

int ikev2_cookie_init(void);
void ikev2_respond_with_cookie(vchar_t *request, struct sockaddr *remote, struct sockaddr *local);
void ikev2_retransmit_add_cookie(struct ikev2_sa *ike_sa, struct ikev2payl_notify *notify);
int ikev2_check_request_cookie(vchar_t *packet, struct sockaddr *remote, struct sockaddr *local);

extern vchar_t * ikev2_auth_calculate(struct ikev2_sa *sa, int i_to_r);
extern int ikev2_auth_verify(struct ikev2_sa *sa, int i_to_r, struct ikev2payl_auth *auth_payload);
extern int ikev2_auth_method(struct ikev2_sa *sa);

vchar_t * ikev2_notify_payload(int protocol_id, u_int8_t *spi, int spi_size,
			       int message_type, u_int8_t *data, size_t data_len);

extern struct prop_pair ** ikev2_parse_sa(struct isakmp_domain *doi, struct ikev2_payload_header *sa);
extern vchar_t * ikev2_construct_sa(struct ikev2_child_sa *child_sa);
extern vchar_t * ikev2_construct_ts_i(struct ikev2_child_sa *child_sa);
extern vchar_t * ikev2_construct_ts_r(struct ikev2_child_sa *child_sa);
extern vchar_t * ikev2_construct_ts(int proto, struct rc_addrlist *addrlist);
extern int ikev2_confirm_ts(struct ikev2_payload_header *ts_payload,
			    int upper_layer_proto,
			    struct rc_addrlist *addrlist);
extern vchar_t * ikev2_identifier(struct rc_idlist *id);

extern int ikev2_compute_keys(struct ikev2_sa *ike_sa);

extern vchar_t * ikev2_prf_plus(struct ikev2_sa *sa, vchar_t *key, vchar_t *msg_bytes, ssize_t need_len);


extern struct prop_pair * ikev2_find_match(struct prop_pair **my_proposal, struct prop_pair **peer_proposal, enum peer_mine which_spi);

extern struct prop_pair * ikev2_get_transforms(struct isakmp_domain *doi, caddr_t payload, struct isakmp_pl_p *prop);
extern int ikev2_compare_transforms(struct isakmp_domain *doi, struct prop_pair *mine, struct prop_pair *peers);
struct prop_pair * ikev2_match_transforms(struct isakmp_domain *, struct prop_pair *, struct prop_pair *);
extern int ikev2_child_compare_transforms(struct isakmp_domain *doi, struct prop_pair *mine, struct prop_pair *peers);
/* struct prop_pair * ikev2_child_match_transforms(struct isakmp_domain *, struct prop_pair *, struct prop_pair *); */

extern struct ikev2_isakmpsa * ikev2_find_match_ikesa(struct rcf_remote *rminfo,
						      struct prop_pair **peer_proposal,
						      isakmp_cookie_t *spi
						      /* , int mode*/
						      );
