/* $Id: utils.h,v 1.12 2005/06/24 03:20:51 mk Exp $ */
/*
 * Copyright (C) 2003 WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef __SPMD_UTILS_H
#define __SPMD_UTILS_H

static inline size_t
__spmd_salen(sa_family_t af)
{
	if (af == AF_INET)
		return (sizeof(struct sockaddr_in));
	else if (af == AF_INET6)
		return (sizeof(struct sockaddr_in6));
	else
		return -1;
}
#define SPMD_SALEN(sa) __spmd_salen( (sa)->sa_family )

#define SPMD_L_MIN	0 /* not use */
#define SPMD_L_ERR	1
#define SPMD_L_WARNING	2
#define SPMD_L_NOTICE	3
#define SPMD_L_INFO	4
#define SPMD_L_DEBUG	5
#define SPMD_L_DEBUG2	6
#define SPMD_L_MAX	7 /* not use */

#define SPMD_L_DEFLT	3  /* default: ERROR and WARNING will be recorded */

extern int spmd_loglevel; 
const char *__spmd_set_tag(const char *file, int line, const char *func);
int __spmd_log(int priority, const char *tag, const char *msg, ...);
#define SPMD_LOG_TAG __spmd_set_tag(__FILE__, __LINE__, __FUNCTION__)
/* output simple log */
#if defined(__GNUC__) && __GNUC__ >= 3
# define SPMD_LOG(priority, fmt, ...) __spmd_log((priority), NULL, (fmt), ## __VA_ARGS__)
# define SPMD_LOGV(priority, fmt, ...) __spmd_log((priority), SPMD_LOG_TAG, (fmt), ## __VA_ARGS__)
# else
# define SPMD_LOG(priority, fmt, args...)  __spmd_log(priority, NULL, fmt , ## args)
# define SPMD_LOGV(priority, fmt, args...)  __spmd_log(priority, SPMD_LOG_TAG, fmt , ## args)
#endif 
/* output verbose log */

int sockcmp(struct sockaddr *sa1, struct sockaddr *sa2);


/*---------- memory staff ---------*/
void *spmd_malloc(size_t size);
void *spmd_calloc(size_t size);
char *spmd_strdup(const char *s);
void spmd_free(void *p);
#endif /* __SPMD_UTILS_H */
